# -*- coding: cp1252 -*-
"A simple calculator for the F4LW."

from f4lw import sync_server
from math import sqrt

class Calc:
  def __init__(self):
    self.acc = 0.0
    self.op = None
    self.inputting = True

  def display(self):
    if self.acc is None:
      return " Err"
    res = "%4d" % self.acc
    if len(res) > 4:
      self.inputting = False
      return "Over"
    else:
      return res

  def enter(self):
    self.inputting = False
    if self.op:
      self.acc = self.op(self.acc)
    self.op = None
    
  def input(self, s):
    try:
      if s.isdigit():
        if self.inputting:
          self.acc = 10.0*self.acc + int(s) # to avoid NotImplemented in method-wrapper
        else:
          self.acc = float(s)
          self.inputting = True
      elif s in "+W":
        self.enter()
        self.op = self.acc.__add__
        self.inputting = False
      elif s in "-Y":
        self.enter()
        self.op = self.acc.__sub__
        self.inputting = False
      elif s in "P":
        self.enter()
        self.op = self.acc.__mul__
        self.inputting = False
      elif s in "F":
        self.enter()
        self.op = self.acc.__div__
        self.inputting = False
      elif s == "E":
        self.enter()
      elif s == "\x08" and self.inputting:
        self.acc = int(self.acc / 10)
      elif s == "A":
        self.acc = -self.acc
      elif s == "V":
        self.inputting = False
        self.acc = sqrt(self.acc)
      elif s == "M":
        self.inputting = True
        self.acc = 0.0
    except ValueError:
      self.acc = None
    except ZeroDivisionError:
      self.acc = None
      self.op = None

  def __del__(self):
    pass

def simulate(s):
  "See the effect of the calculator on the input string."
  calc = Calc()
  res = []
  for c in s:
    calc.input(c)
    res.append(calc.display())
  return res

if __name__ == "__main__":
  sync_server(Calc)
